from django.utils.translation import ugettext_lazy as _
from rest_framework.exceptions import ValidationError
from rest_framework.utils.representation import smart_repr
from rest_framework.compat import unicode_to_repr


class RequiredValidator:
    """Checks that all fields listed are present."""
    message = _('This field is required')

    def __init__(self, fields):
        self.fields = fields

    def enforce_required_fields(self, attrs):
        missing = {name: self.message for name in self.fields if name not in attrs}
        if missing:
            raise ValidationError(missing)

    def __call__(self, attrs):
        self.enforce_required_fields(attrs)

    def __repr__(self):
        return unicode_to_repr('<{}(fields={})>'.format(
            self.__class__.__name__, smart_repr(self.fields)))



from rest_framework.decorators import api_view, renderer_classes
from rest_framework.renderers import CoreJSONRenderer
from rest_framework.response import Response
from rest_framework.schemas import SchemaGenerator
from rest_framework_swagger.renderers import SwaggerUIRenderer, OpenAPIRenderer


def get_schema_view(title):
    @api_view()
    @renderer_classes([SwaggerUIRenderer, OpenAPIRenderer, CoreJSONRenderer])
    def schema_view(request):
        return Response(SchemaGenerator(title=title).get_schema(request))

    return schema_view



"""URLs for the API interfaces."""
from itertools import chain

from django.conf import settings
from django.conf.urls import include, url
from rest_auth import urls as rest_auth_urls
from rest_auth.registration import urls as rest_auth_registration_urls
from rest_framework.documentation import include_docs_urls
from rest_framework.routers import DefaultRouter
from rest_framework.viewsets import ViewSetMixin
from rest_framework_jwt import views as jwt_views
from dj_core.utils import get_app_submodules

from .views import get_schema_view


v1_router = DefaultRouter()

# Add viewsets from apps in the format "routes = [(r'regex', MyViewSet), ...]"
viewsets = [
    module.routes for app_name, module in get_app_submodules('routes')
    if app_name != __package__ and hasattr(module, 'routes')
]
for regex, viewset in chain.from_iterable(viewsets):
    if not issubclass(viewset, ViewSetMixin):
        raise TypeError('Only ViewSets may be added to routers')
    v1_router.register(regex, viewset, base_name=regex)

api_v1 = [
    url(r'', include(v1_router.urls)),
    url(r'^docs/', get_schema_view(title=settings.DJCORE.SITE_NAME)),
    url(r'^drf-docs/', include_docs_urls(title=settings.DJCORE.SITE_NAME)),
    url(r'^auth/', include(rest_auth_urls)),
    url(r'^auth/token/', include([
        url(r'obtain/', jwt_views.obtain_jwt_token),
        url(r'refresh/', jwt_views.refresh_jwt_token),
        url(r'verify/', jwt_views.verify_jwt_token),
    ])),
]

if settings.ACCOUNT_REGISTRATION.lower() == 'enabled':
    api_v1.append(url(r'^auth/registration/', include(rest_auth_registration_urls)))


urlpatterns = [url(r'^backend/api/v1/', include(api_v1))]



#!/usr/bin/env python
import io

from setuptools import setup, find_packages


with io.open('README.md', encoding='utf-8') as handle:
    readme = handle.read()


with io.open('requirements-production.txt', encoding='utf-8') as handle:
    requirements = [line.strip('\n').strip() for line in handle.readlines()]


setup(
    name='dj-core-drf',
    version='0.0.3',
    description='A self-contained and extensible Django Rest Framework environment',
    long_description=readme,
    author='Ionata Digital',
    author_email='webmaster@ionata.com.au',
    url='https://github.com/ionata/dj-core-drf',
    packages=find_packages('src'),
    install_requires=requirements,
    package_dir={'': 'src'},
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)
